### Analyse data files from TRANSPIROMETER
### (code by Nico Blüthgen)

### For detailed description see Mangels et al. 2017 - supplementary material S1

rm(list=ls())
#setwd("~/Desktop/R Respi Transpi")
setwd("~/Documents/Jule/2016 - TranspiRespi/Paper/Appendix/Transpi")

##################### CHOOSE NAMES FOR FILES HERE!!!!! #####################
File1 <- "example_E.txt" ## Name of raw data file from transpirometer
File2 <- "example_E_Tab.csv"  ## Name of table with basic sample data
File3 <- "example_E_Results.csv" ## Give a name for the Results file 
File4 <- "example_E_Results.pdf" ## Give a name for the pdf with Figures 
pdf(File4) ### Store all figures as pdf (skip if not required)

#####################
calibration <- read.delim(file=File1, sep=";", skip = 1, nrows=11, header=FALSE) # read first rows of original data file
names(calibration) <- c("Title1","Offset_V","Title2","Slope")
calibration # calibration data per chamber
calibr_info <- read.delim(file=File1, sep=";", skip = 32, nrows=5, header=FALSE) # read second part of original data file
calibr_info # calibration information
transpi0 <- read.delim(file=File1, sep=";", skip=52) # read measurement data (last part of original data file)
transpi0[,1] <- as.integer(as.character(transpi0[,1])) # 1st column is otherwise corrupted
str(transpi0) # summarize data
transpi0$Air_RH = transpi0$Air_T = transpi0$Box_T = transpi0$X = transpi0$Date =  transpi0$Time<- NULL

Temp <- 25                      ### --> °C Temperature
airflow <- 20                   ### --> airflow as adjusted at the flowmeter [ml/min]
airflow <- airflow*60/1000000   ### --> converted airflow for calculation [m3/h]
TimeStep <- 300                 ### --> ms time intervals for measurement, e.g. 1 row = 200 ms (see calibr_info)
REFStartH <- 500                 ### IMPORTANT --> Define StartH row for reference measurement (null probe)
REFStopH <- 1200                 ### IMPORTANT --> Define StopH row for reference measurement (null probe)

data <- read.table(file=File2, header=T, sep=";", dec=".") # read summary data table
StartH <- data$StartH # StartH row for calculation (all samples)
StopH <- data$StopH   # StopH row for calculation
REFtranspi <- transpi0[c(REFStartH:REFStopH),]  # Only the selected reference time (null probe)

## Reference humidity
Offset_V <- calibration$Offset_V[1:10]
Slope <- calibration$Slope[1:10]
REFMean <- array(data=NA, dim=10)
for (i in 1:10) { 
  REFMean[i] <- mean(as.numeric(REFtranspi[,i]), na.rm=TRUE)
}
REF_V <- REFMean * (5/1024) # Volt
REF_rH <- (REF_V - Offset_V) / Slope # Rel. humidity

#####################
## Measurement humidity
Raw <- transpi0[,1:10] # select only the 10 columns for the 10 chambers
V <- Raw * (5/1024) # Volt 
rHobj <- rH <- array(data=NA, dim=c(dim(Raw)[1], 10))
for (i in 1:10) { 
  rHobj[,i] <- (V[,i] - Offset_V[i]) / Slope[i] # Rel. humidity measured
  rH[,i] <- rHobj[,i] - REF_rH[i]               # Rel. humidity gain (difference)
}

### saturation vapour pressure (Ew)
SaturationPressure_Pa <- 6.1078*10^(7.5*Temp/(237.3+Temp)) # Magnus-formula
### actual vapour pressure (E)
ActualPressure_Pa <- rH/100*SaturationPressure_Pa
### absolute humidity (aH)
aH <- (10^5)*18.016/8314.3*ActualPressure_Pa/(237.5+Temp)
### water loss (Lw)
Waterloss_mg <- aH*airflow*1000

#####################
REFMean <- RawMean <- rH_Mean <- Waterloss_Mean <- Waterloss_SD <- Waterloss_Median <- Waterloss_Min <- Waterloss_Max <- Waterloss_N <- Input <- TimeSpan <- Waterloss_Absolute <- array(data=NA, dim=10)

for (i in 1:10) { 
  REFMean[i] <- mean(as.numeric(REFtranspi[,i]), na.rm=TRUE) # mean raw values of reference measurement (control)
  Waterloss_N[i] <- StopH[i]-StartH[i] # number of measurements considered
  RawMean[i] <- mean(as.numeric(Raw[StartH[i]:StopH[i],i]), na.rm=TRUE) # mean raw data
  rH_Mean[i] <- mean(as.numeric(rH[StartH[i]:StopH[i],i]), na.rm=TRUE) # mean relative humidity
  Waterloss_Mean[i] <- mean(as.numeric((Waterloss_mg[StartH[i]:StopH[i],i]), na.rm=TRUE)) # mean water loss mg/h
  Waterloss_SD[i] <- sd(as.numeric((Waterloss_mg[StartH[i]:StopH[i],i]), na.rm=TRUE))     # SD water loss mg/h
  Waterloss_Median[i] <- median(as.numeric((Waterloss_mg[StartH[i]:StopH[i],i]), na.rm=TRUE)) # median water loss mg/h
  Waterloss_Min[i] <- min(as.numeric((Waterloss_mg[StartH[i]:StopH[i],i]), na.rm=TRUE))   # min water loss mg/h
  Waterloss_Max[i] <- max(as.numeric((Waterloss_mg[StartH[i]:StopH[i],i]), na.rm=TRUE))   # max water loss mg/h
  Input[i] <- which(Raw[,i] == max(Raw[,i], na.rm=TRUE))[1] # row where max (animal put into chamber), 
  TimeSpan[i] <- (dim(Raw)[1] - Input[i])*TimeStep/1000/60/60 # Time in hours from input to end for calculating estimated total mg water loss
  Waterloss_Absolute[i] <- Waterloss_Mean[i]*TimeSpan[i] # Total water loss from StartH to end in mg
}

Output = data.frame(data$No, data$group, data$site, data$trap_type, data$fresh_weight,  data$dry_weight, Waterloss_Mean, 
                    Waterloss_SD, Waterloss_Median, Waterloss_Min, Waterloss_Max, Offset_V, Slope, StartH, 
                    StopH, Waterloss_N, Input, TimeSpan, Waterloss_Absolute, REFStartH, REFStopH, REFMean, RawMean, rH_Mean)
Output
write.table(Output, File3, sep=";")

group <- data$group

### FOR FILTERING: RAW DATA all measurements for selecting appropriate time limits for reference and water loss measurement  
### One Plot with all samples along entire measurement
MaxY <- max(Raw, na.rm=T) ## Define maximum transpiration measured
MinY <- min(Raw, na.rm=T) ## Define minimum transpiration measured
plot(Raw[,1], type="n", xlab="Time (Data rows)", ylab="Transpiration raw", ylim=c(MinY,MaxY))
abline(v=REFStartH, col="black", lty=2)
abline(v=REFStopH, col="black", lty=2)
abline(v=min(StartH), col="black", lty=3)
abline(v=max(StopH), col="black", lty=3)
for (s in 1:10) {     ## Loop through samples
  points(Raw[,s], col=s, pch=20)
}
### One Plot per sample along entire measurement 
for (s in 1:10) {     ## Loop through samples
  plot(Raw[,s], xlab="Time (Data rows)", ylab="Transpiration raw", ylim=c(MinY,MaxY), main=paste("No",s,group[s]), pch=20, col="red")
  abline(v=REFStartH, col="black", lty=2)
  abline(v=REFStopH, col="black", lty=2)  
  abline(v=StartH[s], col="black", lty=3)
  abline(v=StopH[s], col="black", lty=3)
}

### Same plotted as Humidity gain
### One Plot with all samples along entire measurement
MaxY <- max(rH, na.rm=T) ## Define maximum transpiration measured
MinY <- min(rH, na.rm=T) ## Define minimum transpiration measured
TimeMin <- 1:length(rH[,1])
TimeMin <- TimeMin*TimeStep/1000/60  # TimeStep = 200ms
for (s in 1:10) {     ## Loop through samples
  plot(rH[,s]~TimeMin, main=paste("No",s,group[s]), xlab="Time [min]", ylab="Relative humidity gain [%]", ylim=c(MinY,MaxY), pch=20, col="red")
  abline(v=REFStartH, col="black", lty=2)
  abline(v=REFStopH, col="black", lty=2)  
  abline(v=StartH[s], col="black", lty=3)
  abline(v=StopH[s], col="black", lty=3)
}

### RESULTS: One Plot with all samples: Calculated water loss rates
MaxY <- max(Waterloss_mg[min(StartH):max(StopH),], na.rm=T) ## Define maximum for plot: highest respiration measured
MinY <- min(Waterloss_mg[min(StartH):max(StopH),], na.rm=T) ## Define maximum for plot: highest respiration measured
TimeMin <- 1:length(Waterloss_mg[min(StartH):max(StopH),1])
TimeMin <- TimeMin*TimeStep/1000/60
plot(Waterloss_mg[min(StartH):max(StopH),1]~TimeMin, type="n", xlab="Time [min]", ylab="Water loss [mg/h]", ylim=c(MinY,MaxY))
for (s in 1:10) {     ## Loop through samples
  points(Waterloss_mg[,s], col=s, pch=20)
}
### One Plot per sample 
for (s in 1:10) {     ## Loop through samples
  TimeMin <- 1:length(Waterloss_mg[StartH[s]:StopH[s],s])
  TimeMin <- TimeMin*TimeStep/1000/60
  MaxYs <- max(Waterloss_mg[StartH[s]:StopH[s],s], na.rm=T) ## Define maximum for plot: highest respiration measured
  MinYs <- min(Waterloss_mg[StartH[s]:StopH[s],s], na.rm=T) ## Define maximum for plot: highest respiration measured
  plot(Waterloss_mg[StartH[s]:StopH[s],s]~TimeMin, main=paste("No",s,group[s]), xlab="Time [min]", ylab="Water loss [mg/h]", ylim=c(MinYs,MaxYs), pch=20, col="red")
}
### Zoom into the centre to look for oscillations etc.
HalfTime <- (max(StopH)-min(StartH))/2
for (s in 1:10) {     ## Loop through samples
  plot(Waterloss_mg[,s] , main=paste("Zoom",s,group[s]), xlab="Time (Data rows)", ylab="Water loss [mg/h]", xlim=c(HalfTime-200, HalfTime+200), pch=20, col="red")
}

dev.off() # skip if figures not stored as pdf

